function [R,Rz,Uz,I,L,E,Z] = zazem(datoteka,zapis,Xrange,Yrange)
% ZAZEM Resavanje na zazemjuvac so proizvolna forma.
%
%   [R,Rz,Uz,I,L,E] = zazem(datoteka,zapis,Xrange,Yrange)
%
%   Vlez:
%    datoteka : Ime na datotekata vo koja se dadeni podatocite za
%               zazemjuvacot. Detali za nejzina sodrzina moze da se vidat
%               vo zazemformat.
%       zapis : Izbor za zapisuvanje na rezultatite vo datoteka:
%               0 - Rezultatite ne se zapisuvaat.
%               1 - Zapisuvanje vo izlezna datoteka cij prv del od imeto e
%               ednakov na promenlivata datoteka, a nastavkata e txt.
%               Ako promenlivata zapis ne se zadade se smeta deka taa e 0.
%      Xrange : Opseg na vrednosti za x koordinati vo format -10:0.2:10 so
%               koi se definira oblasta za prebaruvanje pri presmetka na
%               potencijalnata razlika na cekor.
%               Ako promenlivata Xrange ne se zadade togas taa ke bide
%               odredena so funkcijata def_oblast.
%      Yrange : Opseg na vrednosti za y koordinati vo format -10:0.2:10 so
%               koi se definira oblasta za prebaruvanje pri presmetka na
%               potencijalnata razlika na cekor.
%               Ako promenlivata Yrange ne se zadade se zema Yrange=Xrange.
%
%   Izlez:
%       R  : Matrica na sopstveni i megusebni otpornosti.
%       Rz : Otpornost na rasprostiranje na zazemjuvacot.
%       Uz : Potencijal na zazemjuvacot.
%       I  : Vektor so strui na odveduvanje od oddelnite sprovodnici na
%            zazemjuvacot.
%       L  : Matrica so podatoci za sprovodnicite. Vo prvite 6 koloni se
%            dadeni x, y i z koordinatite na krajnite tocki od sekoj
%            sprovodnik (izrazeni vo m), a vo kolonata 7 e dijametarot na
%            sprovodnikot (izrazen vo mm).
%       E  : Struktura so polinja Ed_max, Ed_min, Ed_sr, Ec_max, Ec_min,
%            Ec_sr vo koja se dadeni maksimalnata, minimalnata i srednata
%            vrednost na potencijalnite razliki na dopri i cekor.
%            Vrednostite se izrazeni vo %.
%
%  See also ZAZEMFORMAT, NOJMAN, POTENCIJAL_TOCKA, POTENCIJAL_LINIJA, POTENCIJAL_INKA, POTENCIJAL_3D, DEF_OBLAST.

% Citanje na podatocite za zazemjuvacot
if isstruct(datoteka)
    Z = datoteka;
else
    Z = feval(datoteka);
end

% matrica na sprovodnicite
L = Z.objekti;
n = size(L,1);

switch nargin
    case 1
        zapis = 0;
        [Xrange,Yrange] = def_oblast(L);
    case 2
        [Xrange,Yrange] = def_oblast(L);
    case 3
        Yrange = Xrange;
end

% koordinati na pocetnite i krajnite tocki na sprovodnicite
[P,Q] = deal(L(:,1:3),L(:,4:6));
P = num2cell(P,2);
Q = num2cell(Q,2);

% dolzina na sprovodnicite
for i = 1:n
    d(i,1) = norm(P{i}-Q{i});
end

% likovi na sprovodnicite so smenet znak pred koordinatata z
L = [L; L];
L(n+1:end,[3 6]) = -L(n+1:end,[3 6]);
d = [d; d];

% Presmetka na vektorot na sopstveni otpori
dp = L(1:n,7)/1000;
Rs = Z.rho./(2*pi*d(1:n)).*log(2*d(1:n)./dp);

% presmetka na matricata na megusebni otpori
R = zeros(n,2*n);
for i = 1:n
  for j = i+1:2*n
      f = @(t) nojman(t, i, j, L);
      R(i,j) = Z.rho/(4*pi*d(j))*integral(f,0,1);
      if j<= n
          R(j,i) = R(i,j);
      end
  end
end
R = R(:,1:n) + R(:,n+1:end) + diag(Rs);

% Otpornost na rasprostiranje na zazemjuvacot
if isfield(Z,'decimali')
    m = 10^Z.decimali;
    R = round(m*R)/m;
    G = R^-1;
    G = round(m*G)/m;
else
    G = R^-1;
end
Rz = 1/sum(sum(G));
Uz = Rz*Z.Iz;

% Raspredelba na strujata na odveduvanje po elementite od zazemjuvacot
I = Uz * sum(G,2);

L = L(1:n,:);
% Napon na dopir
izlez = extendPoly(Z.ograda,1,2*pi/360); % prosiri ja ogradata za 1 m
ograda1 = izlez{1,1}; % izlez e cellarray
[xd,yd] = deal(ograda1(:,1),ograda1(:,2));
nd = length(xd);
Ed = zeros(nd,1);
for i = 1:nd
    Ed(i) = Uz - 2*potencijal_tocka([xd(i) yd(i) 0],I,L,Z.rho);
end
Ed = Ed/Uz*100;
E.Ed_max = max(Ed);
E.Ed_min = min(Ed);
E.Ed_sr = mean(Ed);

% Napon na cekor
[X, Y] = meshgrid(Xrange,Yrange);
[k, m] = size(X);
fi = zeros(k,m);
for i = 1:k
    for j = 1:m
        fi(i,j) = 2*potencijal_tocka([X(i,j) Y(i,j) 0],I,L,Z.rho);
    end
end
vo = inpolygon(X,Y,Z.ograda(:,1),Z.ograda(:,2));
DX = Xrange(2)-Xrange(1);
DY = Yrange(2)-Yrange(1);
[Gx, Gy] = gradient(fi,DX,DY);
Ec =  sqrt(Gx.^2 + Gy.^2)/Uz*100;
Ec = Ec(~vo);
E.Ec_max = max(Ec(:));
E.Ec_min = min(Ec(:));
E.Ec_sr = mean(Ec(:));

if zapis == 1

    fid = fopen([datoteka '.txt'],'w');

    % zapisuvanje na resenieto
    fprintf(fid,'Rz = %.3f Omi\n', Rz);
    fprintf(fid,'Uz = %.3f V\n', Uz);
    fprintf(fid,'Ed_max = %7.3f %%\n', E.Ed_max);
    fprintf(fid,'Ed_min = %7.3f %%\n', E.Ed_min);
    fprintf(fid,'Ed_sr  = %7.3f %%\n', E.Ed_sr);
    fprintf(fid,'Ec_max = %7.3f %%\n', E.Ec_max);
    fprintf(fid,'Ec_min = %7.3f %%\n', E.Ec_min);
    fprintf(fid,'Ec_sr  = %7.3f %%\n', E.Ec_sr);
    fprintf(fid,'\nMatrica R (vo poslednata kolona e I(A))\n');
    for i = 1:n
        for j = 1:n
            fprintf(fid,'%8.3f',R(i,j));
        end
        fprintf(fid,'%10.3f\n',I(i));
    end

    % presmetka i zapisuvanje na potencijalite vo zadadenite tocki
    if isfield(Z,'tocki')
        fprintf(fid,'\nPotencijali vo zadadenite tocki\n');
        fprintf(fid,'   x(m)   y(m)   z(m)   fi(V)   fi(%%)\n');
        [xt,yt,zt] = deal(Z.tocki(:,1), Z.tocki(:,2), Z.tocki(:,3));
        nt = length(xt);
        fi = zeros(nt,1);
        for i = 1:nt
            fi(i) = 2*potencijal_tocka([xt(i) yt(i) zt(i)],[I;I],L,Z.rho);
        end
        fprintf(fid,'%7.2f%7.2f%7.2f%8.2f%8.2f\n',[xt yt zt fi fi/Uz*100]');
    end

    % presmetka i zapisuvanje na potencijalite po zadadenite linii
    if isfield(Z,'linii')
        fprintf(fid,'\nPotencijali po zadadenite linii\n');
        [x1,y1,x2,y2,delta] = ...
            deal(Z.linii(:,1),Z.linii(:,2),Z.linii(:,3),Z.linii(:,4),Z.linii(:,5));
        nl = length(x1);
        for i = 1:nl
            [x1i,y1i,x2i,y2i,deltai] = ...
                deal(x1(i),y1(i),x2(i),y2(i),delta(i));
            fprintf(fid,'\nLinija %i: (%.2f,%.2f) -- (%.2f,%.2f)\n',i,x1i,y1i,x2i,y2i);
            fprintf(fid,'   x(m)   y(m)   L(m)   fi(V)   fi(%%)\n');
            dx = x2i - x1i;
            dy = y2i - y1i;
            dl = sqrt(dx^2 + dy^2);
            nt = 1 + round(dl/deltai);
            xt = zeros(nt,1); yt = xt; fi = xt;
            for k = 1:nt
                xt(k) = x1i + (k - 1) * dx / (nt - 1);
                yt(k) = y1i + (k - 1) * dy / (nt - 1);
                fi(k) = 2*potencijal_tocka([xt(k) yt(k) 0],[I;I],L,Z.rho);
            end
            dt = sqrt((xt - x1i).^2 + (yt - y1i).^2);
            fprintf(fid,'%7.2f%7.2f%7.2f%8.2f%8.2f\n',[xt yt dt fi fi/Uz*100]');
        end
    end
    fclose(fid);
end
